class Base:
    def __init__(self, stage, username=None, key=None):
        self.__version__ = "0.3.9"

        if stage == "develop":
            self.headers = {"Authorization": username + "," + key}
            self.url = "https://test-api.dcentralab.com/"
            self.web3index_url = "https://test-api.web3index.info/"
        if stage == "staging":
            self.headers = {"Authorization": username + "," + key}
            self.url = "https://staging.dcentralab.com/"
            self.web3index_url = "https://staging-api.web3index.info/"
        if stage == "preprod":
            self.headers = {"Authorization": username + "," + key}
            self.url = "https://preprod-api.dcentralab.com/"
            self.web3index_url = "https://preprod-api.web3index.info/"
        if stage == "main":
            self.headers = {"Authorization": username + "," + key}
            self.url = "https://main-api.dcentralab.com/"
            self.web3index_url = "https://api.web3index.info/"
        if stage == "staging-api":
            self.headers = {"Authorization": username + "," + key}
            self.url = "https://staging-api.dcentralab.com/"
            self.web3index_url = "https://staging-api.web3index.info/"


class DapiError:
    def __init__(self, response, exception):
        self.response = response
        self.exception = exception



from typing import Union, List, Dict
from dcentrapi.requests_dappi import requests_get, requests_post
from dcentrapi.Base import Base


class EventPolling(Base):
    def get_collection(self, collection_name: str):
        url = self.url + "event_polling/collection"
        data = {"collection_name": collection_name}
        response = requests_post(url=url, json=data, headers=self.headers)
        return response.json()

    def get_latest_contract_version(self, contract_name: str):
        url = self.url + "event_polling/get_latest_contract_version"
        data = {"contract_name": contract_name}
        response = requests_get(url=url, params=data, headers=self.headers)
        return response.json()

    def register_user(self, user_name: str, collection_name: str):
        url = self.url + "event_polling/register_user"
        data = {
            "user_name": user_name,
            "collection_name": collection_name,
        }
        response = requests_post(url=url, json=data, headers=self.headers)
        return response.json()

    def subscribe_contract(
        self,
        contract_name: str,
        contract_address: str,
        network: str,
        abi: Union[str, List[Dict[str, str]]],
        collection_name: str,
        contract_version: str,
        git_tag: str = None,
        webhook_url: str = None,
    ):
        url = self.url + "event_polling/subscribe_contract"
        data = {
            "contract_name": contract_name,
            "contract_address": contract_address,
            "contract_version": contract_version,
            "network": network,
            "collection_name": collection_name,
            "abi": abi,
            "git_tag": git_tag,
            "webhook_url": webhook_url,
        }
        response = requests_post(url=url, json=data, headers=self.headers)
        return response.json()

    def get_schema(self, contract_name: str, contract_version: str):
        url = self.url + "event_polling/schema"
        data = {"contract_name": contract_name, "contract_version": contract_version}
        response = requests_get(url=url, params=data, headers=self.headers)
        return response.json()

    def get_events_sum_of_values_in_range(
        self,
        collection_name: str,
        contract_address: str,
        event_name: str,
        field_name: str,
        start_time: str,
        end_time: str,
    ):
        url = self.url + "event_polling/events_sum_of_values_in_range"
        data = {
            "collection_name": collection_name,
            "contract_address": contract_address,
            "event_name": event_name,
            "field_name": field_name,
            "start_time": start_time,
            "end_time": end_time,
        }
        response = requests_get(url=url, params=data, headers=self.headers)
        return response.json()

    def get_contract_nof_transactions(self, collection_name: str, contract_address: str):
        url = self.url + "event_polling/contract_nof_transactions"
        data = {"collection_name": collection_name, "contract_address": contract_address}
        response = requests_get(url=url, params=data, headers=self.headers)
        return response.json()

    def get_contract_users_in_time_range(
        self, collection_name: str, contract_address: str, start_time: str, end_time: str
    ):
        url = self.url + "event_polling/contract_users_in_time_range"
        data = {
            "collection_name": collection_name,
            "contract_address": contract_address,
            "start_time": start_time,
            "end_time": end_time,
        }
        response = requests_get(url=url, params=data, headers=self.headers)
        return response.json()

    def get_contracts_events_info(
        self,
        collection_name: str,
        contract_addresses: [str],
        event_names: [str],
        user_web3_addresses=None,
        start_time=None,
        end_time=None,
        incentive_id: str = None,
        user_address: str = None,
        sort: str = None,
    ):
        url = self.url + "event_polling/contracts_events_info"
        data = {
            "collection_name": collection_name,
            "contract_addresses": contract_addresses,
            "start_time": start_time,
            "end_time": end_time,
            "user_web3_addresses": user_web3_addresses,
            "event_names": event_names,
            "incentive_id": incentive_id,
            "user_address": user_address,
            "sort": sort,
        }
        response = requests_post(url=url, json=data, headers=self.headers)
        return response.json()

    def get_nof_contracts_events_unique_transactions(
        self, collection_name: str, contract_addresses: str, event_names: str
    ):
        url = self.url + "event_polling/contracts_events_info"
        data = {
            "collection_name": collection_name,
            "contract_addresses": contract_addresses,
            "event_names": event_names,
        }
        response = requests_get(url=url, params=data, headers=self.headers)
        return response.json()

    def get_nof_token_transfers(self, contract_addresses: [str]):
        url = self.url + "event_polling/token_transfers"
        data = {"contract_addresses": contract_addresses}
        response = requests_get(url=url, params=data, headers=self.headers)
        return response.json()

    def get_event_details(self, collection_name: str, list_of_events: [str], event_parameter: str):
        url = self.url + "event_polling/get_event_details"
        data = {
            "collection_name": collection_name,
            "list_of_events": list_of_events,
            "event_parameter": event_parameter,
        }
        response = requests_post(url=url, json=data, headers=self.headers)
        return response.json()

    # def get_collection_sum_of_values_in_range(
    #     self, collection_name: str, event_name: str, field_name: str, start_time: str, end_time: str
    # ):
    #     url = self.url + "event_polling/collection_sum_of_values_in_range"
    #     data = {
    #         "collection_name": collection_name,
    #         "event_name": event_name,
    #         "field_name": field_name,
    #         "start_time": start_time,
    #         "end_time": end_time,
    #     }
    #     response = requests_get(url=url, params=data, headers=self.headers)
    #     return response.json()

    # def get_collection_contracts_sum_of_values(
    #     self, collection_name: str, event_name: str, field_name: str, start_time: str, end_time: str
    # ):
    #     url = self.url + "event_polling/collection_contracts_sum_of_values"
    #     data = {
    #         "collection_name": collection_name,
    #         "event_name": event_name,
    #         "field_name": field_name,
    #         "start_time": start_time,
    #         "end_time": end_time,
    #     }
    #     response = requests_get(url=url, params=data, headers=self.headers)
    #     return response.json()

    # def get_collection_daily_nof_transactions(self, collection_name: str, start_time: str, end_time: str):
    #     url = self.url + "event_polling/collection_daily_nof_transactions"
    #     data = {"collection_name": collection_name, "start_time": start_time, "end_time": end_time}
    #     response = requests_get(url=url, params=data, headers=self.headers)
    #     return response.json()

    # def get_collection_nof_transactions_by_time(self, collection_name, start_time, end_time):
    #     url = self.url + "event_polling/collection_nof_transactions_by_time"
    #     data = {"collection_name": collection_name, "start_time": start_time, "end_time": end_time}
    #     response = requests_get(url=url, params=data, headers=self.headers)
    #     return response.json()

    # def get_collection_nof_transactions(self, collection_name: str):
    #     url = self.url + "event_polling/collection_nof_transactions"
    #     data = {"collection_name": collection_name}
    #     response = requests_get(url=url, params=data, headers=self.headers)
    #     return response.json()

    # def get_collection_nof_users_in_time_range(self, collection_name: str, start_time: str, end_time: str):
    #     url = self.url + "event_polling/collection_nof_users_in_time_range"
    #     data = {"collection_name": collection_name, "start_time": start_time, "end_time": end_time}
    #     response = requests_get(url, params=data, headers=self.headers)
    #     return response.json()

    # def get_collection_users_in_time_range(self, collection_name: str, start_time: str, end_time: str):
    #     url = self.url + "event_polling/collection_users_in_time_range"
    #     data = {"collection_name": collection_name, "start_time": start_time, "end_time": end_time}
    #     response = requests_get(url=url, params=data, headers=self.headers)
    #     return response.json()



import requests


def requests_get(url, params=None, headers=None):
    return requests.get(url=url, headers=headers, params=params)


def requests_post(url, json=None, headers=None):
    return requests.post(url=url, headers=headers, json=json)



from dcentrapi.Base import Base
from dcentrapi.requests_dappi import requests_get, requests_post


class RpcAggregation(Base):
    def get_token_balance(self, user, token, network, rpc_url=None):
        url = self.url + "tokenBalance"
        data = {"network": network, "user": user, "token": token, "rpc_url": rpc_url}
        response = requests_get(url, params=data, headers=self.headers)
        return response.json()

    def get_token_balances_for_user(self, user, tokens: list, network, rpc_url=None):
        url = self.url + "tokenBalancesForUser"
        data = {"network": network, "user": user, "tokens": tokens, "rpc_url": rpc_url}
        response = requests_post(url, json=data, headers=self.headers)
        return response.json()

    def get_token_balance_for_users(self, users: list, token, network, rpc_url=None):
        url = self.url + "tokenBalanceForUsers"
        data = {"network": network, "users": users, "token": token, "rpc_url": rpc_url}
        response = requests_post(url, json=data, headers=self.headers)
        return response.json()

    def calculate_token_price_from_pair(self, pool, target_token_address, network, rpc_url=None):
        url = self.url + "calculateTokenPriceFromPair"
        data = {"network": network, "lp_token": pool, "target_token_address": target_token_address, "rpc_url": rpc_url}
        response = requests_get(url, params=data, headers=self.headers)
        return response.json()

    def calculate_reserves_amount_from_pair(self, pool, amount, network, rpc_url=None):
        url = self.url + "calculateReservesAmountsFromPair"
        data = {"network": network, "lp_token": pool, "amount": amount, "rpc_url": rpc_url}
        response = requests_get(url, params=data, headers=self.headers)
        return response.json()

    def get_reserves_from_pair(self, pool, network, rpc_url=None):
        url = self.url + "getReservesFromPair"
        data = {"network": network, "lp_token": pool, "rpc_url": rpc_url}
        response = requests_get(url, params=data, headers=self.headers)
        return response.json()

    # Accepted values for factory_type (May 2023):
    # UNI_V2, UNI_V3, CURVE, BALANCER

    def get_reserves_from_pools(self, factory_type, pools: list, network, rpc_url=None):
        url = self.url + "getReservesFromPools"
        data = {
            "factory_type": factory_type,
            "pools": pools,
            "network": network,
            "rpc_url": rpc_url
        }
        response = requests_post(url, json=data, headers=self.headers)
        return response.json()

    # Same accepted factory_type values as for get_reserves_from_pools above
    def get_pool_data_from_factory(self, factory_type, factory, indices: list, network, rpc_url=None):
        url = self.url + "getPoolDataFromFactory"
        data = {
            "factory_type": factory_type,
            "factory": factory,
            "indices": indices,
            "network": network,
            "rpc_url": rpc_url
        }
        response = requests_post(url, json=data, headers=self.headers)
        return response.json()



from dcentrapi.Base import Base
from dcentrapi.Base import DapiError
from dcentrapi.eventPolling import EventPolling
from dcentrapi.rpcAggregation import RpcAggregation
from dcentrapi.txSimulation import TxSimulation
from dcentrapi.gasMonitor import GasMonitor
from dcentrapi.web3Index import Web3Index
from dcentrapi.hackMitigation import HackMitigation
from dcentrapi.requests_dappi import requests_get, requests_post
# from dcentrapi.merkleTree import MerkleTree, MerkleTreeNode



from typing import List
from dcentrapi.Base import Base
from dcentrapi.requests_dappi import requests_post

SIM_TYPE_QUICK = 'quick'
SIM_TYPE_ABI = 'abi'
SIM_TYPE_FULL = 'full'


# See: https://docs.tenderly.co/simulations-and-forks/intro-to-simulations
class TxSimulation(Base):

    # Format of tx:
    # tx = {
    #     "from": "0x1234",
    #     "to": "0x5678",
    #     "input": "0x12345678000000...data",
    # }
    # Optional parameters:
    # "gas": 1234567,
    # "gas_price": 0,
    # "value": 100,
    # "state_objects": (see Tenderly documentation for overwriting state)

    # See: https://docs.tenderly.co/simulations-and-forks/simulation-api/using-simulation-api
    def simulate_transaction_single(
        self,
        tx: dict,
        network_id: int,  # chain-id
        simulation_type: str = SIM_TYPE_FULL,  # quick, abi, or full
    ):
        url = self.url + "simulate_transactions"
        data = {
            "simulation_type": simulation_type,
            "network_id": network_id,
            "tx_single": tx,
        }
        response = requests_post(url, json=data, headers=self.headers)
        return response.json()

    # Format of tx_bundle is [tx0, tx1, tx2...]
    # See: https://docs.tenderly.co/simulations-and-forks/simulation-api/simulation-bundles
    def simulate_transaction_bundle(
        self,
        tx_bundle: List[dict],
        network_id: int,
        simulation_type: str = SIM_TYPE_FULL,
    ):
        url = self.url + "simulate_transactions"
        data = {
            "simulation_type": simulation_type,
            "network_id": network_id,
            "tx_bundle": tx_bundle,
        }
        response = requests_post(url, json=data, headers=self.headers)
        return response.json()
